@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Wavefront SDK Counter.
 
 @author Hao Song (songhao@vmware.com)
@@ -21,6 +20,11 @@ class WavefrontSdkCounter(metrics.WavefrontSdkMetric):
         with self._lock:
             self._count += val
 
+    def dec(self, val=1):
+        """Decrease the value of the counter."""
+        with self._lock:
+            self._count -= val
+
     def count(self):
         """Get the value of the counter."""
         with self._lock:
